//+------------------------------------------------------------------+
//|                                               Heiken Ashiper.mq4 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 6

//--- Heikin Ashi Properties
#property indicator_color1 Black
#property indicator_width1 1
#property indicator_color2 Gray
#property indicator_width2 1
#property indicator_color3 Black
#property indicator_width3 3
#property indicator_color4 Gray
#property indicator_width4 3

//--- Arrow Properties
#property indicator_color5 Green
#property indicator_width5 3
#property indicator_color6 Red
#property indicator_width6 3

//--- User Inputs
extern color downshadow=clrBlack;    // Shadow of bear candlestick
extern color upshadow=clrGray;  // Shadow of bull candlestick
extern color downbody=clrBlack;    // Bear candlestick body
extern color upbody=clrGray;  // Bull candlestick body

extern color uparrowcolor=clrGreen;
extern color downarrowcolor=clrRed;

//--- Arrays, Variables and Buffers
double lowhighb[];
double highlowb[];
double openb[];
double closeb[];
double uparrow[];
double downarrow[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   IndicatorShortName("Heiken Ashi");
   IndicatorDigits(Digits);
//--- indicator buffers mapping
   SetIndexBuffer(0,lowhighb);
   SetIndexStyle(0,DRAW_HISTOGRAM,0,1,downshadow);
   SetIndexLabel(0,"Low/High");
   SetIndexBuffer(1,highlowb);
   SetIndexStyle(1,DRAW_HISTOGRAM,0,1,upshadow);
   SetIndexLabel(1,"High/Low");
   SetIndexBuffer(2,openb);
   SetIndexStyle(2,DRAW_HISTOGRAM,0,3,downbody);
   SetIndexLabel(2,"Open");
   SetIndexBuffer(3,closeb);
   SetIndexStyle(3,DRAW_HISTOGRAM,0,3,upbody);
   SetIndexLabel(3,"Close");

   SetIndexDrawBegin(0,10);
   SetIndexDrawBegin(1,10);
   SetIndexDrawBegin(2,10);
   SetIndexDrawBegin(3,10);

//--- Arrow buffers mapping
   SetIndexBuffer(4,uparrow);
   SetIndexStyle(4,DRAW_ARROW,0,3,uparrowcolor);
   SetIndexArrow(4,233);
   SetIndexLabel(4,"Bullish haChange");

   SetIndexBuffer(5,downarrow);
   SetIndexStyle(5,DRAW_ARROW,0,3,downarrowcolor);
   SetIndexArrow(5,234);
   SetIndexLabel(5,"Bearish haChange");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//--- Wholesale taken from default Heikin Ashi
   int    i,pos;
   double haOpen,haHigh,haLow,haClose;

//---
   if(rates_total<=10)
      return(0);
//--- counting from 0 to rates_total
   ArraySetAsSeries(lowhighb,false);
   ArraySetAsSeries(highlowb,false);
   ArraySetAsSeries(openb,false);
   ArraySetAsSeries(closeb,false);
   ArraySetAsSeries(open,false);
   ArraySetAsSeries(high,false);
   ArraySetAsSeries(low,false);
   ArraySetAsSeries(close,false);
//--- preliminary calculation
   if(prev_calculated>1)
      pos=prev_calculated-1;
   else
     {
      //--- set first candle
      if(open[0]<close[0])
        {
         lowhighb[0]=low[0];
         highlowb[0]=high[0];
        }
      else
        {
         lowhighb[0]=high[0];
         highlowb[0]=low[0];
        }
      openb[0]=open[0];
      closeb[0]=close[0];
      //---
      pos=1;
     }
//--- main loop of calculations
   for(i=pos; i<rates_total; i++)
     {
      haOpen=(openb[i-1]+closeb[i-1])/2;
      haClose=(open[i]+high[i]+low[i]+close[i])/4;
      haHigh=MathMax(high[i],MathMax(haOpen,haClose));
      haLow=MathMin(low[i],MathMin(haOpen,haClose));

      if(haOpen<haClose)
        {
         lowhighb[i]=haLow;
         highlowb[i]=haHigh;
        }
      else
        {
         lowhighb[i]=haHigh;
         highlowb[i]=haLow;
        }
      openb[i]=haOpen;
      closeb[i]=haClose;
     }
//--- drawing done

//--- Comparison and signal
//--- Past to Present For Loop
   int counted_bars=IndicatorCounted();
   int limit= Bars-counted_bars;
   for(int k=(Bars-100);k>limit;k--)
     {
      //if gray then black
      if(openb[k+1]<closeb[k+1] && openb[k]>closeb[k])
         downarrow[k]=closeb[k];
      //else if black then gray
      else if(openb[k+1]>closeb[k+1] && openb[k]<closeb[k])
         uparrow[k]=closeb[k];
     }

//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---

  }
//+------------------------------------------------------------------+
